@@ -1,7 +1,7 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:ft=tcl:et:sw=4:ts=4:sts=4
 #
 # Copyright (c) 2002 - 2003 Apple Inc.
-# Copyright (c) 2004 - 2013 The MacPorts Project
+# Copyright (c) 2004 - 2016, 2018 The MacPorts Project
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -38,6 +38,7 @@ set org.macports.archivefetch [target_new org.macports.archivefetch portarchivef
 #target_init ${org.macports.archivefetch} portarchivefetch::archivefetch_init
 target_provides ${org.macports.archivefetch} archivefetch
 target_requires ${org.macports.archivefetch} main
+target_runtype ${org.macports.archivefetch} always
 target_prerun ${org.macports.archivefetch} portarchivefetch::archivefetch_start
 
 namespace eval portarchivefetch {
@@ -150,7 +151,7 @@ proc portarchivefetch::get_full_archive_sites_path {} {
 proc portarchivefetch::checkfiles {urls} {
     upvar $urls fetch_urls
 
-    portfetch::checksites [list archive_sites [list {} {} ARCHIVE_SITE_LOCAL]] \
+    portfetch::checksites [list archive_sites [list {} ARCHIVE_SITE_LOCAL]] \
                           [get_full_archive_sites_path]
     checkarchivefiles fetch_urls
 }
@@ -295,7 +296,7 @@ proc portarchivefetch::fetchfiles {args} {
     if {[info exists archive_exists]} {
         # modify state file to skip remaining phases up to destroot
         global target_state_fd
-        foreach target {fetch checksum extract patch configure build destroot} {
+        foreach target {archivefetch fetch checksum extract patch configure build destroot} {
             write_statefile target "org.macports.${target}" $target_state_fd
         }
         return 0
