@@ -4,38 +4,38 @@
  * Some basic readline support callable from Tcl
  * By James D. Berry <jberry@macports.org> 10/27/05
  *
- * $Id: readline.c 26177 2007-06-15 10:11:22Z jmpp@macports.org $
- *
  */
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#if HAVE_STDLIB_H
-#include <stdlib.h>
-#endif
-
-#if HAVE_STRING_H
-#include <string.h>
-#endif
+/* required for strdup(3) on Linux and macOS */
+#define _BSD_SOURCE
 
 #include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <errno.h>
 
-#if HAVE_READLINE_READLINE_H
+#ifdef HAVE_READLINE_READLINE_H
 #include <readline/readline.h>
 #endif
 
-#if HAVE_READLINE_HISTORY_H
+#ifdef HAVE_READLINE_HISTORY_H
 #include <readline/history.h>
 #endif
 
+#if HAVE_SYS_FILE_H
+#include <sys/file.h>
+#endif
+
 #include <tcl.h>
+
 #include "readline.h"
 
-
 /* Globals */
-#if HAVE_READLINE_READLINE_H
+#ifdef HAVE_READLINE_READLINE_H
 Tcl_Interp* completion_interp = NULL;
 Tcl_Obj* attempted_completion_word = NULL;
 Tcl_Obj* generator_word = NULL;
@@ -61,7 +61,7 @@ Tcl_Obj* generator_word = NULL;
 #endif
 
 
-#if HAVE_LIBREADLINE
+#ifdef HAVE_LIBREADLINE
 char*
 completion_generator(const char* text, int state)
 {
@@ -162,8 +162,10 @@ int ReadlineCmd(ClientData clientData UNUSED, Tcl_Interp *interp, int objc, Tcl_
 {
 	char* action;
 	Tcl_Obj *tcl_result;
+#ifdef HAVE_LIBREADLINE
 	int argbase;
 	int argcnt;
+#endif
 	
 	/* Get the action */
 	if (objc < 2) {
@@ -177,7 +179,7 @@ int ReadlineCmd(ClientData clientData UNUSED, Tcl_Interp *interp, int objc, Tcl_
 	
 		int initOk = 0;
 		
-#if HAVE_LIBREADLINE
+#ifdef HAVE_LIBREADLINE
 		/* Set the name of our program, so .inputrc can be conditionalized */
 		if (objc == 3) {
 			rl_readline_name = strdup(Tcl_GetString(objv[2]));
@@ -198,7 +200,7 @@ int ReadlineCmd(ClientData clientData UNUSED, Tcl_Interp *interp, int objc, Tcl_
 		tcl_result = Tcl_NewIntObj(initOk);	
 		Tcl_SetObjResult(interp, tcl_result);
 
-#if HAVE_LIBREADLINE
+#ifdef HAVE_LIBREADLINE
 	} else if (0 == strcmp("read", action)) {
 	
 		char* s;
@@ -270,15 +272,19 @@ int ReadlineCmd(ClientData clientData UNUSED, Tcl_Interp *interp, int objc, Tcl_
 		add line
 		read filename
 		write filename
+		append filename
 		stifle max
 		unstifle
 */
 int RLHistoryCmd(ClientData clientData UNUSED, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[])
 {
 	char* action = NULL;
+#ifdef HAVE_LIBREADLINE
 	char* s = NULL;
 	int i = 0;
 	Tcl_Obj *tcl_result;
+	FILE *hist_file;
+#endif
 
 	if (objc < 2) {
 		Tcl_WrongNumArgs(interp, 1, objv, "action");
@@ -288,7 +294,7 @@ int RLHistoryCmd(ClientData clientData UNUSED, Tcl_Interp *interp, int objc, Tcl
 
 	/* Case out on action */
 	if (0) {
-#if HAVE_LIBREADLINE
+#ifdef HAVE_LIBREADLINE
 	} else if (0 == strcmp("add", action)) {
 		if (objc != 3) {
 			Tcl_WrongNumArgs(interp, 1, objv, "add line");
@@ -310,6 +316,37 @@ int RLHistoryCmd(ClientData clientData UNUSED, Tcl_Interp *interp, int objc, Tcl
 		}
 		s = Tcl_GetString(objv[2]);
 		write_history(s);
+	}  else if (0 == strcmp("append", action)) {
+		/* Begin code for history appension
+		 *
+		 * This action is a bit more convoluted than the others, so I think it
+		 * will benefit from some comments */
+
+		/* Just like the other cases, make sure the arg count in Tcl is correct */
+		if (objc != 3) {
+			Tcl_WrongNumArgs(interp, 1, objv, "append filename");
+			return TCL_ERROR;
+		}
+
+		/* Open the file supplied in Tcl; return an error if it fails */
+		s = Tcl_GetString(objv[2]);
+		if (NULL == (hist_file = fopen(s, "a"))) {
+			Tcl_AppendResult(interp, "fopen(", s, "): ", strerror(errno), NULL);
+			return TCL_ERROR;
+		}
+
+		/* Lock the history file, write the line to a buffer (catching errors),
+		 * flush it, unlock the file and close it */
+		flock(fileno(hist_file), LOCK_EX);
+		if (fprintf(hist_file, "%s\n", current_history()->line) < 0) {
+			Tcl_AppendResult(interp, "fprintf(", current_history()->line, "): ", strerror(errno), NULL);
+			return TCL_ERROR;
+		}
+		fflush(hist_file);
+		flock(fileno(hist_file), LOCK_UN);
+		fclose(hist_file);
+
+		/* End code for history appension */
 	} else if (0 == strcmp("stifle", action)) {
 		if (objc != 3) {
 			Tcl_WrongNumArgs(interp, 1, objv, "stifle maxlines");
